@@ -30,6 +30,7 @@
 #include "util.h"
 #include "../common/xchat.h"
 #include "../common/xchatc.h"
+#include "../common/dcc.h"
 
 enum {
 	DCC_COLUMN,
@@ -51,10 +52,11 @@ dcc_window_class_init (DccWindowClass *klass)
 static void
 transfer_selection_changed (GtkTreeSelection *selection, DccWindow *window)
 {
-	if (gtk_tree_selection_get_selected (selection, NULL, NULL))
+	if (gtk_tree_selection_get_selected (selection, NULL, NULL)) {
 		gtk_widget_set_sensitive (window->stop_button, TRUE);
-	else
+	} else {
 		gtk_widget_set_sensitive (window->stop_button, FALSE);
+	}
 }
 
 static void
@@ -66,13 +68,13 @@ transfer_stop_clicked (GtkButton *button, DccWindow *window)
 	struct DCC *dcc;
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (window->transfer_list));
-	if (gtk_tree_selection_get_selected (selection, &model, &iter) == FALSE)
+	if (gtk_tree_selection_get_selected (selection, &model, &iter) == FALSE) {
 		return;
+	}
 
 	gtk_tree_model_get (model, &iter, DCC_COLUMN, &dcc, -1);
-	dcc_abort (dcc->serv->server_session, dcc);
-
 	gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+	dcc_abort (dcc->serv->server_session, dcc);
 }
 
 static void
@@ -80,14 +82,15 @@ dcc_window_init (DccWindow *window)
 {
 	GladeXML *xml;
 	GtkTreeSelection *selection;
+	gchar *path;
 
 	xml = NULL;
-	if (g_file_test ("../../data/dcc-window.glade", G_FILE_TEST_EXISTS))
-		xml = glade_xml_new ("../../data/dcc-window.glade", "toplevel", NULL);
-	if (!xml)
-		xml = glade_xml_new (XCHATSHAREDIR "/dcc-window.glade", "toplevel", NULL);
-	if (!xml)
+	path = locate_data_file ("dcc-window.glade");
+	xml = glade_xml_new (path, "toplevel", NULL);
+	g_free (path);
+	if (!xml) {
 		return;
+	}
 
 #define GW(name) ((window->name) = glade_xml_get_widget (xml, #name))
 	GW(transfer_list);
@@ -116,6 +119,7 @@ dcc_window_init (DccWindow *window)
 
 	gtk_window_set_default_size (GTK_WINDOW (window), 300, 400);
 	gtk_window_set_title (GTK_WINDOW (window), _("File Transfers"));
+	g_signal_connect (G_OBJECT (window), "key-press-event", G_CALLBACK (dialog_escape_key_handler_hide), NULL);
 
 	window->progress_column = gtk_tree_view_column_new ();
 	window->info_column = gtk_tree_view_column_new ();
@@ -153,13 +157,9 @@ dcc_window_init (DccWindow *window)
 	gtk_tree_view_append_column (GTK_TREE_VIEW (window->transfer_list), window->info_column);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (window->transfer_list), window->remaining_column);
 
-	if (g_file_test ("../../data/stock_up.png", G_FILE_TEST_EXISTS)) {
-		window->up_icon   = gdk_pixbuf_new_from_file_at_size ("../../data/stock_up.png",   24, 24, NULL);
-		window->down_icon = gdk_pixbuf_new_from_file_at_size ("../../data/stock_down.png", 24, 24, NULL);
-	} else {
-		window->up_icon   = gdk_pixbuf_new_from_file_at_size (XCHATSHAREDIR "/stock_up.png",   24, 24, NULL);
-		window->down_icon = gdk_pixbuf_new_from_file_at_size (XCHATSHAREDIR "/stock_down.png", 24, 24, NULL);
-	}
+	GtkIconTheme *theme = gtk_icon_theme_get_default ();
+	window->up_icon   = gtk_icon_theme_load_icon (theme, "go-up",   24, 0, NULL);
+	window->down_icon = gtk_icon_theme_load_icon (theme, "go-down", 24, 0, NULL);
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (window->transfer_list));
 	g_signal_connect (G_OBJECT (selection), "changed", G_CALLBACK (transfer_selection_changed), window);
@@ -188,13 +188,6 @@ dcc_window_get_type (void)
 	return dcc_window_type;
 }
 
-static gboolean
-dcc_window_delete_event (GtkWidget *window, GdkEvent *event, gpointer data)
-{
-	gtk_widget_hide (window);
-	return TRUE;
-}
-
 DccWindow *
 dcc_window_new (void)
 {
@@ -204,7 +197,7 @@ dcc_window_new (void)
 		return NULL;
 	}
 
-	g_signal_connect (G_OBJECT (window), "delete-event", G_CALLBACK (dcc_window_delete_event), NULL);
+	g_signal_connect (G_OBJECT (window), "delete-event", G_CALLBACK (gtk_widget_hide_on_delete), NULL);
 
 	return window;
 }
@@ -217,36 +210,22 @@ dcc_window_add (DccWindow *window, struct DCC *dcc)
 	gchar *done_text, *info_text;
 	char *size, *pos;
 
-	/* We don't care about DCC chats */
-	if (dcc->type == 2)
-		return;
-
 	/* If this is a recieve and auto-accept isn't turned on, pop up a
 	 * confirmation dialog */
-	if ((dcc->type == 1) && (dcc->dccstat == 0) && (prefs.autodccsend == FALSE)) {
-		GladeXML *xml = NULL;
-		GtkWidget *dialog, *label;
-		GtkResponseType response;
-		gchar *text;
+	if ((dcc->type == TYPE_RECV) && (dcc->dccstat == STAT_QUEUED) && (prefs.autodccsend == FALSE)) {
+		GtkWidget *dialog;
+		gint response;
 
-		if (g_file_test ("../../dcc-accept.glade", G_FILE_TEST_EXISTS))
-			xml = glade_xml_new ("../../dcc-accept.glade", "toplevel", NULL);
-		if (!xml)
-			xml = glade_xml_new (XCHATSHAREDIR "/dcc-accept.glade", "toplevel", NULL);
-		if (!xml)
-			return;
-
-		dialog = glade_xml_get_widget (xml, "toplevel");
-		label  = glade_xml_get_widget (xml, "label");
-
-		g_object_unref (xml);
-
-		text = g_strdup_printf (_("<span weight=\"bold\" size=\"larger\">Incoming File Transfer</span>\n\n%s is attempting to send you a file named \"%s\".  Do you wish to accept the transfer?"), dcc->nick, dcc->file);
-		gtk_label_set_markup (GTK_LABEL (label), text);
-		g_free (text);
+		dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+						 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+						 GTK_MESSAGE_QUESTION,
+						 GTK_BUTTONS_CANCEL,
+						 _("Incoming File Transfer"));
+		gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Accept"), GTK_RESPONSE_ACCEPT);
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), _("%s is attempting to send you a file named \"%s\". Do you wish to accept the transfer?"), dcc->nick, dcc->file);
 
 		response = gtk_dialog_run (GTK_DIALOG (dialog));
-		if (response == GTK_RESPONSE_OK) {
+		if (response == GTK_RESPONSE_ACCEPT) {
 			/* FIXME: resume? */
 			dcc_get (dcc);
 
@@ -286,6 +265,11 @@ dcc_window_update (DccWindow *window, struct DCC *dcc)
 {
 	GtkTreeIter iter;
 
+	/* DCC chats cause crashes here, because they're not handled in this window. */
+	if (dcc->type == 2) {
+		return;
+	}
+
 	if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (window->transfer_store), &iter)) {
 		do {
 			gpointer ptr;
@@ -306,12 +290,12 @@ dcc_window_update (DccWindow *window, struct DCC *dcc)
 				g_free (pos);
 				g_free (speed);
 
-				if (dcc->dccstat == 0) {
+				if (dcc->dccstat == STAT_QUEUED) {
 					remaining_text = g_strdup (_("queued"));
-				} else if (dcc->dccstat == 2) {
+				} else if (dcc->dccstat == STAT_FAILED) {
 					gchar *message;
 
-					if (dcc->type == 0)
+					if (dcc->type == TYPE_SEND)
 						message = g_strdup_printf (_("Transfer of %s to %s failed"), dcc->file, dcc->nick);
 					else
 						message = g_strdup_printf (_("Transfer of %s from %s failed"), dcc->file, dcc->nick);
@@ -319,9 +303,11 @@ dcc_window_update (DccWindow *window, struct DCC *dcc)
 					g_free (message);
 					gtk_list_store_remove (window->transfer_store, &iter);
 					return;
-				} else if (dcc->dccstat == 3) {
+				} else if (dcc->dccstat == STAT_DONE) {
 					gtk_list_store_remove (window->transfer_store, &iter);
 					return;
+				} else if (dcc->dccstat == STAT_ABORTED) {
+					remaining_text = g_strdup (_("aborted"));
 				} else {
 					if (dcc->cps == 0) {
 						remaining_text = g_strdup (_("stalled"));
@@ -335,11 +321,11 @@ dcc_window_update (DccWindow *window, struct DCC *dcc)
 				}
 
 				gtk_list_store_set (window->transfer_store, &iter,
-						    INFO_COLUMN, info_text,
-						    DONE_COLUMN, done,
-						    DONE_LABEL_COLUMN, done_text,
-						    TIME_COLUMN, remaining_text,
-						    -1);
+				                    INFO_COLUMN, info_text,
+				                    DONE_COLUMN, done,
+				                    DONE_LABEL_COLUMN, done_text,
+				                    TIME_COLUMN, remaining_text,
+				                    -1);
 				g_free (done_text);
 				g_free (info_text);
 				g_free (remaining_text);
@@ -347,7 +333,7 @@ dcc_window_update (DccWindow *window, struct DCC *dcc)
 				/* We put off rendering the icon until we get at least one update,
 				 * to ensure that gnome-vfs can determine the MIME type
 				 */
-				if (icon == NULL) {
+				if (dcc->destfile != NULL && icon == NULL) {
 					GtkIconTheme *icon_theme;
 					char *icon;
 					char *mime;
@@ -356,8 +342,9 @@ dcc_window_update (DccWindow *window, struct DCC *dcc)
 					gpointer image_data;
 
 					/* If the file doesn't exist yet, don't do the icon */
-					if (g_file_test (dcc->destfile, G_FILE_TEST_EXISTS) == FALSE)
+					if (g_file_test (dcc->destfile, G_FILE_TEST_EXISTS) == FALSE) {
 						return;
+					}
 
 					/* Get MIME type from gnome-vfs, create the proper file icon for it. */
 					mime = gnome_vfs_get_mime_type (dcc->destfile);
@@ -385,10 +372,11 @@ dcc_window_update (DccWindow *window, struct DCC *dcc)
 					                      GDK_INTERP_BILINEAR, 255);
 
 					/* Composite on an emblem which shows the direction of transfer */
-					if (dcc->type == 1)
+					if (dcc->type == 1) {
 						direction_emblem = window->down_icon;
-					else
+					} else {
 						direction_emblem = window->up_icon;
+					}
 
 					gdk_pixbuf_composite (direction_emblem,
 					                      file_icon,
@@ -425,8 +413,10 @@ void
 dcc_window_remove (DccWindow *window, struct DCC *dcc)
 {
 	GtkTreeIter iter;
-	if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (window->transfer_store), &iter) == FALSE)
+	if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (window->transfer_store), &iter) == FALSE) {
 		return;
+	}
+
 	do {
 		gpointer ptr;
 		gtk_tree_model_get (GTK_TREE_MODEL (window->transfer_store), &iter, DCC_COLUMN, &ptr, -1);
@@ -443,12 +433,15 @@ dcc_send_file (struct User *user)
 	GtkWidget *dialog;
 	GtkResponseType response;
 
+	if (user->nick == NULL)
+		return;
+
 	dialog = gtk_file_chooser_dialog_new (_("Send File..."),
-			GTK_WINDOW (gui.main_window),
-			GTK_FILE_CHOOSER_ACTION_OPEN,
-			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-			GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
-			NULL);
+	                GTK_WINDOW (gui.main_window),
+	                GTK_FILE_CHOOSER_ACTION_OPEN,
+	                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+	                GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
+	                NULL);
 
 	response = gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_hide (dialog);
