@@ -95,7 +95,6 @@ fe_args (int argc, char *argv[])
 
 	context = g_option_context_new (NULL);
 
-#ifdef HAVE_LIBGNOME_214
 	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
 	gui.program = gnome_program_init (PACKAGE, VERSION,
 	                                  LIBGNOMEUI_MODULE, argc, argv,
@@ -103,18 +102,6 @@ fe_args (int argc, char *argv[])
 	                                  GNOME_PARAM_HUMAN_READABLE_NAME, "IRC Chat",
 	                                  GNOME_PROGRAM_STANDARD_PROPERTIES,
 	                                  NULL);
-#else
-	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
-	g_option_context_add_group (context, gtk_get_option_group (TRUE));
-	g_option_context_parse (context, &argc, &argv, &error);
-	g_option_context_free (context);
-
-	gui.program = gnome_program_init (PACKAGE, VERSION,
-	                                  LIBGNOMEUI_MODULE, argc, argv,
-	                                  GNOME_PARAM_HUMAN_READABLE_NAME, "IRC Chat",
-	                                  GNOME_PROGRAM_STANDARD_PROPERTIES,
-	                                  NULL);
-#endif
 
 	if (error) {
 		fprintf (stderr, _("xchat-gnome: %s\nTry `xchat-gnome --help' for more information\n"), error->message);
@@ -193,9 +180,7 @@ fe_main (void)
 {
 	gtk_main ();
 
-#ifdef HAVE_LIBGNOME_214
 	g_object_unref (gui.program);
-#endif
 
 	/* sleep for 3 seconds so any QUIT messages are not lost. The  */
 	/* GUI is closed at this point, so the user doesn't even know! */
