@@ -1,7 +1,7 @@
 /*
  * irc-network.c - an object which interfaces around the core 'ircnet'
  *
- * Copyright (C) 2004-2005 xchat-gnome team
+ * Copyright (C) 2004-2006 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,24 +19,31 @@
  *
  */
 
+#include <config.h>
+#include <glib/gi18n.h>
+#include <string.h>
 #include "irc-network.h"
 #include "navigation-tree.h"
 #include "gui.h"
+#include "../common/server.h"
 
 const char *encodings[] =
 {
-	"UTF-8",
-	"ISO-8859-15 (Western Europe)",
-	"ISO-8859-2 (Central Europe)",
-	"ISO-8859-7 (Greek)",
-	"ISO-8859-8 (Hebrew)",
-	"ISO-8859-9 (Turkish)",
-	"ISO-2022-JP (Japanese)",
-	"SJIS (Japanese)",
-	"CP949 (Korean)",
-	"CP1251 (Cyrillic)",
-	"CP1256 (Arabic)",
-	"GB18030 (Chinese)",
+	N_("UTF-8 (Unicode)"),
+	N_("ISO-8859-15 (Western Europe)"),
+	N_("ISO-8859-2 (Central Europe)"),
+	N_("ISO-8859-7 (Greek)"),
+	N_("ISO-8859-8 (Hebrew)"),
+	N_("ISO-8859-9 (Turkish)"),
+	N_("ISO-2022-JP (Japanese)"),
+	N_("SJIS (Japanese)"),
+	N_("CP949 (Korean)"),
+	N_("KOI8-R (Cyrillic)"),
+	N_("CP1251 (Cyrillic)"),
+	N_("CP1256 (Arabic)"),
+	N_("CP1257 (Baltic)"),
+	N_("GB18030 (Chinese)"),
+	N_("TIS-620 (Thai)"),
 	NULL
 };
 
@@ -50,12 +57,14 @@ irc_network_finalize (GObject *object)
 	GSList *s;
 
 	g_free (network->name);
+	g_free (network->nickserv_password);
 	g_free (network->password);
 	g_free (network->nick);
 	g_free (network->real);
 
-	for (s = network->servers; s; s = g_slist_next (s))
+	for (s = network->servers; s; s = g_slist_next (s)) {
 		g_free (((ircserver *) s->data)->hostname);
+	}
 	g_slist_foreach (network->servers, (GFunc) g_free, NULL);
 	g_slist_free (network->servers);
 
@@ -122,6 +131,7 @@ irc_network_new (ircnet *net)
 	n->name        = g_strdup (net->name);
 	n->autoconnect = net->flags & FLAG_AUTO_CONNECT;
 	n->use_ssl     = net->flags & FLAG_USE_SSL;
+	n->allow_invalid = net->flags & FLAG_ALLOW_INVALID;
 	n->cycle       = net->flags & FLAG_CYCLE;
 
 	for (s1 = net->servlist; s1; s1 = g_slist_next (s1)) {
@@ -131,11 +141,13 @@ irc_network_new (ircnet *net)
 	}
 	n->servers = s2;
 
+	n->nickserv_password = g_strdup (net->nickserv);
 	n->password    = g_strdup(net->pass);
-	if (net->encoding)
+	if (net->encoding) {
 		n->encoding = GPOINTER_TO_UINT (g_hash_table_lookup (enctoindex, net->encoding));
-	else
+	} else {
 		n->encoding = 0;
+	}
 	n->use_global  = net->flags & FLAG_USE_GLOBAL;
 	n->nick        = g_strdup (net->nick);
 	n->real        = g_strdup (net->real);
@@ -152,6 +164,7 @@ irc_network_save (IrcNetwork *network)
 	ircnet *net = network->net;
 	guint32 flags = 0;
 	GSList *s;
+	struct server *serv;
 
 	if (net == NULL) {
 		net = servlist_net_add (network->name, "", TRUE);
@@ -161,34 +174,49 @@ irc_network_save (IrcNetwork *network)
 	if (net->pass)     g_free (net->pass);
 	if (net->nick)     g_free (net->nick);
 	if (net->real)     g_free (net->real);
+	if (net->nickserv) g_free (net->nickserv);
 	if (net->autojoin) g_free (net->autojoin);
 	if (net->encoding) g_free (net->encoding);
 
-	net->name     = g_strdup (network->name);
-	net->pass     = g_strdup (network->password);
-	net->nick     = g_strdup (network->nick);
-	net->real     = g_strdup (network->real);
+	net->name = g_strdup (network->name);
+	if (network->password && strlen (network->password) != 0) {
+		net->pass = g_strdup (network->password);
+	} else {
+		net->pass = NULL;
+	}
+	net->nick = g_strdup (network->nick);
+	net->real = g_strdup (network->real);
+	if (network->nickserv_password && strlen (network->nickserv_password) != 0) {
+		net->nickserv = g_strdup (network->nickserv_password);
+	} else {
+		net->nickserv = NULL;
+	}
 	net->autojoin = g_strdup (network->autojoin);
 	net->encoding = g_strdup (encodings[network->encoding]);
 
-	struct server *serv = navigation_model_get_server (gui.tree_model, net);
-	if (serv)
+	serv = navigation_model_get_server (gui.tree_model, net);
+	if (serv) {
 		server_set_encoding (serv, net->encoding);
+	}
 
 	if (network->autoconnect) flags |= FLAG_AUTO_CONNECT;
 	if (network->use_ssl)     flags |= FLAG_USE_SSL;
+	if (network->allow_invalid) flags |= FLAG_ALLOW_INVALID;
 	if (network->cycle)       flags |= FLAG_CYCLE;
 	if (network->use_global)  flags |= FLAG_USE_GLOBAL;
 	net->flags = flags;
 
 	while (net->servlist) {
 		ircserver *is = net->servlist->data;
-		if (is && is->hostname)
+		if (is && is->hostname) {
 			servlist_server_remove (net, is);
+		}
 	}
 	for (s = network->servers; s; s = g_slist_next (s)) {
 		ircserver *is = s->data;
-		servlist_server_add (net, is->hostname);
+		if (!(is == NULL || is->hostname == NULL)) {
+			servlist_server_add (net, is->hostname);
+		}
 	}
 
 	servlist_save ();
