@@ -1,7 +1,7 @@
 /*
  * pixmaps.h - helper functions for pixmaps
  *
- * Copyright (C) 2004-2005 xchat-gnome team
+ * Copyright (C) 2004-2006 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,21 +22,6 @@
 #ifndef XCHAT_GNOME_PIXMAPS_H
 #define XCHAT_GNOME_PIXMAPS_H
 
-#include <gdk-pixbuf/gdk-pixbuf.h>
-
-extern GdkPixbuf *pix_purple;
-extern GdkPixbuf *pix_red;
-extern GdkPixbuf *pix_op;
-extern GdkPixbuf *pix_hop;
-extern GdkPixbuf *pix_voice;
-
-extern GdkPixbuf *pix_newdata;
-extern GdkPixbuf *pix_nicksaid;
-extern GdkPixbuf *pix_msgsaid;
-
-extern GdkPixbuf *pix_prefs_plugins;
-extern GdkPixbuf *pix_prefs_keybindings;
-
 extern void pixmaps_init (void);
 
 #endif
