@@ -1,7 +1,7 @@
 /*
  * preferences-page-networks.c - helpers for the servers preferences page
  *
- * Copyright (C) 2004-2005 xchat-gnome team
+ * Copyright (C) 2004-2006 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -62,10 +62,9 @@ add_clicked (GtkWidget *button, PreferencesNetworksPage *page)
 	n = irc_network_new (NULL);
 	e = irc_network_editor_new (n);
 
+	g_object_weak_ref (G_OBJECT (e), (GWeakNotify) populate, page);
 	irc_network_editor_run (e);
-	gtk_widget_destroy (GTK_WIDGET (e));
 	g_object_unref (n);
-	populate (page);
 }
 
 static void
@@ -79,25 +78,26 @@ edit_clicked (GtkWidget *button, PreferencesNetworksPage *page)
 	GtkTreeIter iter;
 
 	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (page->network_list));
-	if (!gtk_tree_selection_get_selected (select, &model, &iter))
+	if (!gtk_tree_selection_get_selected (select, &model, &iter)) {
 		return;
+	}
 	gtk_tree_model_get (model, &iter, 1, &net, -1);
 
 	n = irc_network_new (net);
 	e = irc_network_editor_new (n);
 
+	g_object_weak_ref (G_OBJECT (e), (GWeakNotify) populate, page);
 	irc_network_editor_run (e);
-	gtk_widget_destroy (GTK_WIDGET (e));
 	g_object_unref (n);
-	populate (page);
 }
 
 static void
 edit_activated (GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *column, PreferencesNetworksPage *page)
 {
 	GtkTreeIter iter;
-	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (page->sort_model), &iter, path))
+	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (page->sort_model), &iter, path)) {
 		edit_clicked (NULL, page);
+	}
 }
 
 static void
@@ -111,8 +111,9 @@ remove_clicked (GtkWidget *button, PreferencesNetworksPage *page)
 	gint r;
 
 	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (page->network_list));
-	if (!gtk_tree_selection_get_selected (select, &model, &iter))
+	if (!gtk_tree_selection_get_selected (select, &model, &iter)) {
 		return;
+	}
 	gtk_tree_model_get(model, &iter, 1, &net, -1);
 
 	dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, _("Really remove network \"%s\" and all of its servers?"), net->name);
@@ -163,10 +164,8 @@ preferences_page_networks_new (gpointer prefs_dialog, GladeXML *xml)
 	gtk_widget_set_sensitive (page->network_edit, FALSE);
 	gtk_widget_set_sensitive (page->network_remove, FALSE);
 
-	if (g_file_test ("../../data/servers.png", G_FILE_TEST_EXISTS))
-		page->icon = gdk_pixbuf_new_from_file ("../../data/servers.png", NULL);
-	else
-		page->icon = gdk_pixbuf_new_from_file (XCHATSHAREDIR "/servers.png", NULL);
+	page->icon = gtk_widget_render_icon (page->network_edit, GTK_STOCK_NETWORK, GTK_ICON_SIZE_MENU, NULL);
+
 	gtk_list_store_append (p->page_store, &iter);
 	gtk_list_store_set (p->page_store, &iter, 0, page->icon, 1, _("Networks"), 2, 4, -1);
 
