@@ -36,6 +36,8 @@ static void       text_entry_finalize          (GObject        *object);
 static gboolean   text_entry_key_press         (GtkWidget      *widget,
                                                 GdkEventKey    *event,
                                                 gpointer        data);
+static void       text_entry_grab_focus        (GtkWidget      *widget);
+
 #ifdef HAVE_LIBSEXY
 static gboolean   text_entry_spell_check       (TextEntry      *entry,
                                                 gchar          *text,
@@ -53,9 +55,6 @@ static gboolean   text_entry_tab_complete      (GtkEntry       *entry);
 static void       text_entry_populate_popup    (GtkEntry       *entry,
                                                 GtkMenu        *menu,
                                                 gpointer        data);
-static void       text_entry_selection_changed (GObject        *obj,
-                                                GParamSpec     *pspec,
-                                                gpointer        data);
 
 static gboolean   tab_complete_command         (GtkEntry       *entry);
 static gboolean   tab_complete_nickname        (GtkEntry       *entry,
@@ -73,7 +72,7 @@ static GtkEntryClass       *parent_class = NULL;
 G_DEFINE_TYPE (TextEntry, text_entry, GTK_TYPE_ENTRY);
 #endif
 
-static gchar *selected_text = NULL;
+gpointer *entry_parent_class;
 
 struct _TextEntryPriv
 {
@@ -86,13 +85,16 @@ struct _TextEntryPriv
 static void
 text_entry_class_init (TextEntryClass *klass)
 {
-	GObjectClass *gobject_class;
+	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
-	gobject_class = G_OBJECT_CLASS (klass);
+	entry_parent_class = g_type_class_peek_parent (g_type_class_peek (GTK_TYPE_ENTRY));
 
 	gobject_class->finalize = text_entry_finalize;
+
+	widget_class->grab_focus = text_entry_grab_focus;
 }
 
 static void
@@ -110,8 +112,6 @@ text_entry_init (TextEntry *entry)
 	g_signal_connect_after (G_OBJECT (entry), "key_press_event",         G_CALLBACK (text_entry_key_press),         NULL);
 	g_signal_connect       (G_OBJECT (entry), "activate",                G_CALLBACK (text_entry_activate),          NULL);
 	g_signal_connect       (G_OBJECT (entry), "populate-popup",          G_CALLBACK (text_entry_populate_popup),    NULL);
-	g_signal_connect       (G_OBJECT (entry), "notify::cursor-position", G_CALLBACK (text_entry_selection_changed), NULL);
-	g_signal_connect       (G_OBJECT (entry), "notify::selection-bound", G_CALLBACK (text_entry_selection_changed), NULL);
 #ifdef HAVE_LIBSEXY
 	g_signal_connect_after (G_OBJECT (entry), "word-check",              G_CALLBACK (text_entry_spell_check),       NULL);
 #endif
@@ -149,8 +149,7 @@ text_entry_init (TextEntry *entry)
 
 		g_slist_foreach (languages, (GFunc) g_free, NULL);
 		g_slist_free (languages);
-	}
-	else {
+	} else {
 		if (enable_spellcheck) {
 			/* We use libsexy default languages and set it in gconf */
 			languages = sexy_spell_entry_get_active_languages (SEXY_SPELL_ENTRY (entry));
@@ -180,35 +179,57 @@ text_entry_finalize (GObject *object)
 
 	entry = TEXT_ENTRY (object);
 
-	if (entry->priv->entries)
+	if (entry->priv->entries) {
 		g_hash_table_destroy (entry->priv->entries);
-	if (entry->priv->command_completion)
+	}
+	if (entry->priv->command_completion) {
 		g_completion_free (entry->priv->command_completion);
-	if (entry->priv)
+	}
+	if (entry->priv) {
 		g_free (entry->priv);
+	}
 
-	if (G_OBJECT_CLASS (parent_class)->finalize)
-		G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static gboolean
 text_entry_key_press (GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
+	guint state = event->state & gtk_accelerator_get_default_mod_mask ();
+	gboolean handled = FALSE;
+
 	switch (event->keyval) {
-		case GDK_Down:
+	case GDK_Down:
+		if (state == 0) {
 			text_entry_history_down (GTK_ENTRY (widget));
-			return TRUE;
-		case GDK_Up:
+			handled = TRUE;
+		}
+		break;
+	case GDK_Up:
+		if (state == 0) {
 			text_entry_history_up (GTK_ENTRY (widget));
-			return TRUE;
-		case GDK_Tab:
-			if (event->state & GDK_CONTROL_MASK)
-				break;
-			return text_entry_tab_complete (GTK_ENTRY (widget));
-		default:
-			break;
+			handled = TRUE;
+		}
+		break;
+	case GDK_Tab:
+		if (state  == 0) {
+			handled = text_entry_tab_complete (GTK_ENTRY (widget));
+		}
+		break;
+	default:
+		break;
 	}
-	return FALSE;
+
+	return handled;
+}
+
+static void
+text_entry_grab_focus (GtkWidget *widget)
+{
+	/* GtkEntry's grab_focus selects the contents and therefore
+	 * claims PRIMARY. So we bypass it; see bug #345356 and bug #347067.
+	 */
+	GTK_WIDGET_CLASS (entry_parent_class)->grab_focus (widget);
 }
 
 #ifdef HAVE_LIBSEXY
@@ -218,19 +239,22 @@ text_entry_spell_check (TextEntry *entry, gchar *text, gpointer data)
 	GtkTreeModel *store = GTK_TREE_MODEL (userlist_get_store (u, entry->priv->current));
 	GtkTreeIter iter;
 
-	if (gtk_tree_model_get_iter_first (store, &iter) == FALSE)
+	if (gtk_tree_model_get_iter_first (store, &iter) == FALSE) {
 		return TRUE;
+	}
 	do {
 		gchar *nick;
 		gboolean match = FALSE;
 
 		gtk_tree_model_get (store, &iter, 1, &nick, -1);
-		if (strncmp (text, nick, strlen (nick)) == 0)
+		if (strncmp (text, nick, strlen (nick)) == 0) {
 			match = TRUE;
+		}
 
 		g_free (nick);
-		if (match)
+		if (match) {
 			return FALSE;
+		}
 	} while (gtk_tree_model_iter_next (store, &iter));
 	return TRUE;
 }
@@ -241,8 +265,9 @@ text_entry_activate (GtkWidget *widget, gpointer data)
 {
 	char *entry_text = g_strdup (gtk_entry_get_text (GTK_ENTRY (widget)));
 	gtk_entry_set_text (GTK_ENTRY (widget), "");
-	if (TEXT_ENTRY (widget)->priv->current != NULL)
+	if (TEXT_ENTRY (widget)->priv->current != NULL) {
 		handle_multiline (TEXT_ENTRY (widget)->priv->current, (char *) entry_text, TRUE, FALSE);
+	}
 	g_free (entry_text);
 }
 
@@ -253,8 +278,9 @@ text_entry_history_up (GtkEntry *entry)
 	char *new_line;
 
 	text_entry = TEXT_ENTRY (entry);
-	if (text_entry->priv->current == NULL)
+	if (text_entry->priv->current == NULL) {
 		return;
+	}
 
 	new_line = history_up (&(text_entry->priv->current->history), (char *)entry->text);
 	if (new_line) {
@@ -270,8 +296,9 @@ text_entry_history_down (GtkEntry *entry)
 	char *new_line;
 
 	text_entry = TEXT_ENTRY (entry);
-	if (text_entry->priv->current == NULL)
+	if (text_entry->priv->current == NULL) {
 		return;
+	}
 
 	new_line = history_down (&(text_entry->priv->current->history));
 	if (new_line) {
@@ -290,13 +317,15 @@ text_entry_tab_complete (GtkEntry *entry)
 	text = gtk_entry_get_text (entry);
 	cursor_pos = gtk_editable_get_position (GTK_EDITABLE (entry));
 
-	if (cursor_pos == 0)
+	if (cursor_pos == 0) {
 		return TRUE;
+	}
 
 	/* If we're directly after a space, we have nothing to tab complete */
 	p = g_utf8_offset_to_pointer (text, cursor_pos - 1);
-	if (p[0] == ' ')
+	if (p[0] == ' ') {
 		return TRUE;
+	}
 
 	/* search from cusror backwards to find /, #, ' ' or start */
 	p = g_utf8_offset_to_pointer (text, cursor_pos);
@@ -425,7 +454,7 @@ tab_complete_command (GtkEntry *entry)
 	cursor = gtk_editable_get_position (GTK_EDITABLE (entry));
 	prefix = g_new0 (char, cursor);
 	text = gtk_entry_get_text (entry);
-	strncpy (prefix, &text[1], cursor - 1);
+	prefix = g_ascii_strup (&text[1], cursor - 1);
 	length = strlen (text);
 
 	options = g_completion_complete (text_entry->priv->command_completion, prefix, &new_prefix);
@@ -441,7 +470,6 @@ tab_complete_command (GtkEntry *entry)
 
 		if (length - cursor == 0) {
 			/* at the end of the entry, just insert */
-
 			npt = g_strdup_printf ("/%s ", (char *) options->data);
 			pos = strlen (npt);
 		} else {
@@ -630,29 +658,48 @@ text_entry_new (void)
 void
 text_entry_set_current (TextEntry *entry, struct session *sess)
 {
-	gchar *text;
+	TextEntryPriv *priv = entry->priv;
+	GtkWidget *widget = GTK_WIDGET (entry);
+	GtkEditable *editable = GTK_EDITABLE (entry);
+	GtkClipboard *clipboard;
+	char *selection = NULL, *text = NULL;
+	int start, end;
 
-	if (sess == entry->priv->current)
+	g_return_if_fail (GTK_WIDGET_REALIZED (widget));
+
+	if (sess == priv->current) {
 		return;
-
-	if (sess == NULL) {
-		gtk_entry_set_text (GTK_ENTRY (entry), "");
-	} else {
-		g_hash_table_insert (entry->priv->entries,
-		                     entry->priv->current,
-		                     g_strdup (gtk_entry_get_text (GTK_ENTRY (entry))));
-		text = g_hash_table_lookup (entry->priv->entries, sess);
-		if (text)
-			gtk_entry_set_text (GTK_ENTRY (entry), text);
-		else
-			gtk_entry_set_text (GTK_ENTRY (entry), "");
-		gtk_editable_set_position (GTK_EDITABLE (entry), -1);
 	}
-	entry->priv->current = sess;
 
-	if (selected_text) {
-		gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_PRIMARY),
-		                        selected_text, strlen (selected_text));
+	/* If the entry owns PRIMARY, setting the new text will clear PRIMARY;
+	 * so we need to re-set PRIMARY after setting the text.
+	 * See bug #345356 and bug #347067.
+	 */
+
+	clipboard = gtk_widget_get_clipboard (widget, GDK_SELECTION_PRIMARY);
+	g_assert (clipboard != NULL);
+
+	if (gtk_clipboard_get_owner (clipboard) == G_OBJECT (entry) &&
+	    gtk_editable_get_selection_bounds (editable, &start, &end)) {
+		selection = gtk_editable_get_chars (editable, start, end);
+	}
+
+	if (sess != NULL) {
+		g_hash_table_insert (priv->entries,
+		                     priv->current,
+		                     g_strdup (gtk_entry_get_text (GTK_ENTRY (entry))));
+		text = g_hash_table_lookup (priv->entries, sess);
+	}
+
+	gtk_entry_set_text (GTK_ENTRY (entry), text ? text : "");
+	gtk_editable_set_position (GTK_EDITABLE (entry), -1);
+
+	priv->current = sess;
+
+	/* Restore the selection (note that it's not owned by us anymore!) */
+	if (selection) {
+		gtk_clipboard_set_text (clipboard, selection, strlen (selection));
+		g_free (selection);
 	}
 }
 
@@ -660,8 +707,9 @@ void
 text_entry_remove_session (TextEntry *entry, struct session *sess)
 {
 	g_hash_table_remove (entry->priv->entries, sess);
-	if (sess == entry->priv->current)
+	if (sess == entry->priv->current) {
 		gtk_entry_set_text (GTK_ENTRY (entry), "");
+	}
 }
 
 #ifdef HAVE_LIBSEXY
@@ -689,19 +737,3 @@ enable_spellcheck_changed (GConfClient *client, guint cnxn_id, GConfEntry *gconf
 	}
 }
 #endif
-
-static void
-text_entry_selection_changed (GObject *obj, GParamSpec *pspec, gpointer data)
-{
-	GtkEditable *editable;
-	gint start, end;
-
-	editable = GTK_EDITABLE (obj);
-	if (gtk_editable_get_selection_bounds (editable, &start, &end)) {
-		if (start != end) {
-			if (selected_text)
-				g_free (selected_text);
-			selected_text = gtk_editable_get_chars (editable, start, end);
-		}
-	}
-}
